/* 
    xWinTV using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef WINTVSCREEN_H
#define WINTVSCREEN_H

#include "wintv.h"
#include "xrwevent.h"

// includes needed by qt
#include <kapp.h>
#include <qpixmap.h>
#include <qwidget.h>
#include <qtimer.h>
#include <qdatetm.h> 
#include <qlcdnum.h>
#include <qpopmenu.h>
#include <qlabel.h>

#include <kprogress.h> 

// includes needed by bttv
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>

// additional
#include "channel.h"

#include <asm/types.h>

#ifdef USE_KERNEL_VIDEODEV
#include <linux/videodev.h>
#else
#include <videodev.h>
#endif

// video clip stuff
#include "videoclip.h"



/* ------------------- some definitions ---------------------------------*/

// input norm
#define PAL 0
#define NTSC 1
#define SECAM 2

// input source
#define TUNER 0
#define COMP1 1
#define COMP2 2
#define SVHS  3

// aspect ratio
#define ASPECT_NONE  0
#define ASPECT_16_9  1
#define ASPECT_4_3   2

// some defaults 
#define DCOL 254
#define DBRI 0
#define DHUE 0
#define DCONTRAST 216
#define DNORM  PAL
#define DIN TUNER
#define DFREQ 772
#define DCHANNEL 1

#define PICNAME "./pic"

#define INAKTIVE 0
#define AKTIVE 1

#define NONE 0

#define DEFAULT_WIDTH   320
#define DEFAULT_HEIGHT   240 

#define PAL_WIDTH 768
#define PAL_WIDE_WIDTH 922
#define PAL_HEIGHT 576

#define NTSC_WIDTH 640
#define NTSC_HEIGHT 480

// several delay times 
#define DEFAULT_DELAY 200       // 0.2 sec
#define AUDIOMODE_DELAY 500     // 0.5 sec.
#define INFO_DELAY 3000         // 3 sec.

// audio 
#define AUDIOMODE_AUTODETECT 256
#define CAN_AUDIO_VOLUME     1

// supported video clip formats
#define VIDEOCLIP_AVI 1
#define VIDEOCLIP_PPM 2
#define VIDEOCLIP_RAW 3

// supported shap shot formats
#define SNAPSHOT_PNM  1
#define SNAPSHOT_JPEG 2
#define SNAPSHOT_GIF  3
#define SNAPSHOT_TIFF 4

#define CHINFO_NODISPVAL -1024

#define MAX_CLIPRECS 100

#define VIEWREFRESH_ALLOW      1
#define VIEWREFRESH_VISIBILITY 2

//============================================================================================
struct STRTAB {
    int  nr;
    char *str;
};

static char *device_cap[] = {
    "capture", "tuner", "teletext", "overlay", "chromakey", "clipping",
    "frameram", "scales", "monochrome", "subcapture", NULL
};

static char *device_pal[] = {
    "-", "grey", "hi240", "rgb16", "rgb24", "rgb32", "rgb15",
    "yuv422", "yuyv", "uyvy", "yuv420", "yuv411", "raw",
    "yuv422p", "yuv411p"
};

static struct STRTAB norms[] = {
    {  0, "PAL" },
    {  1, "NTSC" },
    {  2, "SECAM" },
    {  3, "AUTO" },
    { -1, NULL }
};

static struct STRTAB *inputs;

static struct STRTAB audiomode[] = {
    {  0, "auto"    },
    {  1, "mono"    },
    {  2, "stereo"  },
    {  3, "lang1"   },
    {  4, "lang2"   },
    { -1, NULL,     },
};


//========================= class winTVScreenWidget =====================================

class winTVScreenWidget : public QWidget
{
  
  Q_OBJECT

public:
  winTVScreenWidget( QWidget *_parent=0, 
		     const char *name=0,
		     int _capture=0, 
		     const char *_bttvdev=BTTV_DEVICE, 
		     int _bpp=0,int _dual=0,
		     int _freq=DFREQ, 
		     int _input=DNORM, 
		     char *_visualname=0
		     );

  ~winTVScreenWidget();
  
  int win_default_width(void) { return DEFAULT_WIDTH; }
  int win_default_height(void) { return  DEFAULT_HEIGHT; }
  int win_max_width(void) { return max_width; }
  int win_max_height(void) { return max_height; }

  int getScreenSaverState() { return saverState; }
  int getCaptureState() { return captureState; }
  int getCursorState() { return cursorState; }

  int getTVX();
  int getTVY();
  int getTVHeight();
  int getTVWidth();

  int getMuteState() { 
    if (-1 == ioctl(fbttv,VIDIOCGAUDIO,&vaudio[0]))
      warning("ioctl VIDIOCGCAUDIO"); 
    //fatal("ioctl VIDIOCGCAUDIO"); 
    return (vaudio[0].flags & VIDEO_AUDIO_MUTABLE) ? (int)(vaudio[0].flags & VIDEO_AUDIO_MUTE) : -1;
  }
  int getAudioMode() { 
    return audioMode;
  }
  int getVolumeState() {
    if (-1 == ioctl(fbttv,VIDIOCGAUDIO,&vaudio[0]))
      fatal("ioctl VIDIOCGCAUDIO");  
    return (vaudio[0].flags & VIDEO_AUDIO_VOLUME) ? (int)vaudio[0].volume : -1;
  }

  int getCanVolume() { return audioVolumeCan; }

  struct video_capability* getCapCap() { return &vcap; }
  struct video_channel* getCapChannel() { return vchan; }
  struct video_audio* getCapAudio() { return vaudio; }
  struct video_tuner* getCapTuner() { return &vtuner; }
  
  void setTVPopup(QPopupMenu *tvp) { tvPopup=tvp; } 
  void destroyTV();

  long getChannelSignal();

  void setViewRefreshMode( int allow ) {  
    allowViewRefreshVisibility= allow&2?true:false;
    allowViewRefresh= allow&1?true:false; }

  int getViewRefreshMode() { return ((allowViewRefresh?1:0) & (allowViewRefreshVisibility?2:0)); }
    
  void setRefreshDelay( int d ) { viewRefreshDelay=d; }
  int getRefreshDelay() { return viewRefreshDelay; }

  int getShotNumber() { return shotnumber; }

  /* -------------------------------------------------------------------------------------- */
  /* -------------------------------------------------------------------------------------- */
public slots:
  // use this slots to change anything 
  void showChannelInfoSLOT(channel *ch,int val, 
			   int min_val, int max_val, const char *text );
  void showChannelInfoEnableSLOT( int on );

  // capture
  void changeCaptureSLOT(int value);       // toggle capture on/off; init is set by constructor

  // tunig et al
  void changeFrequencySLOT(int value);     // does what its name says ...
  void changeBrightnessSLOT(int bri);      
  void changeContrastSLOT(int contr);
  void changeColorSLOT(int color);
  void changeHueSLOT(int hue);
  void changeSourceSLOT(int s);
  void changeNormSLOT(int n);

  // set all 
  void changeChannelValuesSLOT(channel *c, int id);

   // sound et al
  void toggleAudioMuteSLOT();                 // toggles mute on/off, init is on
  void changeAudioVolumeSLOT(int vl, int vr);     // 
  void changeAudioModeSLOT(int mode);         // init is auto

  // screen saver and mouse cursor
  void toggleScreenSaverSLOT();      // toggles screen saver on/off; init is on
  void toggleMouseCursorSLOT();      // toggles mouse cursor on/off; init is on

  // unused stuff
  void changeTVMultiDisplaySLOT(int x, int y, int w, int h); // not used yet

  // one snap shot
  void snapShotSLOT(const char *prefix, int fixed, int width, int height, int format);

  // create video clip; format: VIDEOCLIP_AVI, VIDEOCLIP_PPM, VIDEOCLIP_RAW
  void startVideoClipSLOT(struct MOVIE_PARAMS *par); 
  void endVideoClipSLOT();

  void postInit();

  void Shellx11EventSLOT(XEvent *);

  /* -------------------------------------------------------------------------------------- */
  /* -----stop reading --- everything below is private ... -------------------------------- */
  /* -------------------------------------------------------------------------------------- */
private slots:
  void refreshTVActionSLOT();
  void autoDetectAudioModeSLOT();
  void writeVideoClipFrameSLOT();
  void unShowChannelInfoSLOT();

  /* -------------------------------------------------------------------------------------- */
signals:
  void keyPressedSIGNAL(int key);   // returns all keys pressed in winTVScreenWidget
  void autoDetectAudioModeSIGNAL(int mode);
  void wroteVideoClipFrameSIGNAL(int frames);
  void lostVideoClipFrameSIGNAL(int frames);
  void statVideoClipFrameSIGNAL(double);
  void toggleTVonlySIGNAL();

  /* -------------------------------------------------------------------------------------- */
protected:
  void resizeEvent( QResizeEvent * );
  void moveEvent( QMoveEvent * );
  void paintEvent (QPaintEvent * );
  //void closeEvent (QCloseEvent * );
  bool eventFilter( QObject *, QEvent * );
  bool x11Event( XEvent * );
  void keyPressEvent ( QKeyEvent *);
  //void mouseMoveEvent (QMouseEvent *);
  void mouseReleaseEvent ( QMouseEvent * me);

  /* -------------------------------------------------------------------------------------- */
private:
  void capture( int on ); // 1=on 0=off
  void switch_capture( int on ); // switch capture on/off 

  int initializeTV( WId parentWid = 0 );

  void newTVGeometry( bool doViewRefresh );
  void setTVMaxSize();

  void refreshTV();

  void addTVClip(struct video_clip *cr,int *ncr,uint x,uint y,uint x2,uint y2);
  void makeTVClipTab(int ncr);
  int getTVClip(void);

  void readData();

/* -------------------------------------------------------------------------------------- */
/* ---------------------------- resources ----------------------------------------------- */
/* -------------------------------------------------------------------------------------- */

  QWidget *parent;

  const char *devname;           // name of the bttv-device
  int fbttv;               // the bttv-device

  // bttv stuff
  bool captureIsOn,captureWasOnQt,captureWasOnX11;          // picture on/off
  int defaultNorm;                // input norm, PAL, NTSC, SECAM
  int defaultFreq;                // frequency, *16 
  int defaultInput;               // input channel, TUNER, COMP1, COMP2, SVHS 
  int viewRefreshDelay;               // 200ms
  int Bright;              // Brightness
  int Color;               // intensity of colors
  int Hue;                 // hue
  int Contrast;            // contrast
  
  bool snap;            // snapshot

  // video 4 linux 
  struct video_window vwin;  // bttv structure 
  struct video_buffer vbuf;  // bttv structure 
  struct video_tuner vtuner; // bttv structure 
  struct video_capability vcap; 
  struct video_channel *vchan;
  struct video_picture vpic;
  struct video_audio *vaudio;
  struct video_clip cliprecs[MAX_CLIPRECS];    // list of clips
  struct video_mmap vgrab_even;
  struct video_mmap vgrab_odd;
  struct video_mbuf vmbuf;
  char *TargetMem;         // memory for snapshot  (was: unchar)
  int msize;               // size of memory for snapshot 

  int max_width;          // max width of TV picture 
  int max_height;         // max height of TV picture, both set by setTVMaxSize

  // color and visual stuff
  //Visual *visual;          // the aktual visual
  char *visname;             // name of visual

  int alloc_context;
  QColor qcolors[256];     // colormap, needed when having only 256 colors -> pseudocolor

  // color depth etc
  bool dual;            // have grafic-card with dual-buffer ( 32 bpp mode ?? )
  int numbitplanes;        // number of bitplanes 
  int bpp;                 // bytes per pixel

  int realw;               // real width of screen (not used yet)
  int aspect;              // aspect ratio (4:3 or 16:9) (not used yet)

  bool colorMap8;              // are we using an own colormap ( <= 256 colors)

  QTimer *viewRefreshTimer;
  bool allowViewRefresh,didViewRefresh,allowViewRefreshVisibility;   
  bool didViewMove;            // the widget was moved

  int widget_is_active;           // the widget is aktive

  QCursor orgCursor;       // Cursor

  QTimer *audioDetectTimer;
  int audioMode,audioVolumeCan,audioVolume;
  
  int cursorState,saverState,muteState,captureState;

  int ss_timeout,ss_interval,ss_prefer_blanking,ss_allow_exposures; // stuff for ScreenSaver

  int shotnumber;          // number of screen shot, starting at 0

  // popup
  QPopupMenu *tvPopup;

  // video clip stuff
  QTime videoTime;
  videoClip *video;
  int video_aktive, video_capturing;
  bool video_frame;
  long elapsed;

  // info tools
  QLCDNumber* chInfoLCD;
  QLabel *chInfoText;
  KProgress *chInfoBar;
  int chInfo;
  QTimer *infoTimer;
  int infodelay;

  // title image
  QPixmap title_pm;
  int title_shown;
};

#endif WINTVSCREEN_H
















